# -*- coding: utf-8 -*-
# @Time    : 26/10/2022 3:41 PM
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : interface_dss.py
# @Software: PyCharm

import py_dss_interface
import opendssdirect as drt


dss = py_dss_interface.DSSDLL()
